package at.bestsolution.rsd.rt.server.ws;

import java.io.IOException;
import java.util.Map;
import java.util.Optional;

import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;

import at.bestsolution.rsd.rt.Delta;
import at.bestsolution.rsd.rt.RSDOject;
import at.bestsolution.rsd.rt.rpc.RPCMessageBuilder;
import at.bestsolution.rsd.rt.rpc.RPCMessageParser;
import at.bestsolution.rsd.rt.rpc.RPCRequestMessage;
import at.bestsolution.rsd.rt.services.SerializationService;
import at.bestsolution.rsd.rt.services.SerializationService.Serializer;

public abstract class BaseWSEndpoint {
	private final RPCMessageParser parser;
	private final RPCMessageBuilder messageBuilder;
	
	private Session session;

	public BaseWSEndpoint(RPCMessageParser parser, RPCMessageBuilder messageBuilder) {
		this.parser = parser;
		this.messageBuilder = messageBuilder;
	}
	
	@OnOpen
	public final void onWebSocketConnect(Session session) {
		this.session = session;
	}
	
	@OnMessage
	public final void receivedMessage(String message) {
		RPCRequestMessage request = parser.request(message);
		try {
			Object value = dispatch(request.method(), request.parameters());
			if( value == Void.class ) {
				session.getBasicRemote().sendText(messageBuilder.response(request.id(), "").message()); 
			} else {
				if( value == null ) {
					session.getBasicRemote().sendText(messageBuilder.response(request.id(), "null").message());
				} else if( value instanceof Number || value instanceof CharSequence ) {
					session.getBasicRemote().sendText(messageBuilder.response(request.id(), value+"").message());
				} else if( value instanceof RSDOject ) {
					session.getBasicRemote().sendText(messageBuilder.response(request.id(), serializeReturnValue(request.method(),(RSDOject) value)).message());
				} else {
					session.getBasicRemote().sendText(messageBuilder.response(request.id(), serializeReturnValue(request.method(),(Delta) value)).message());
				}
			}	
		} catch(Throwable t) {
			t.printStackTrace();
			try {
				session.getBasicRemote().sendText(messageBuilder.responseError(request.id(), t.getMessage()).message());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}
	
	protected abstract Object dispatch(String method, Map<String, String> parameters);
	protected abstract String serializeReturnValue(String method, Object value);
}
